/*
 *  Constantin Mihalache <mihalache.c94@gmail.com>
 *  Grupa: 333CB
 */

#include "menu.h"
#include "lcd.h"

#include <util/atomic.h>

enum MENU_STATE {
    STATE_WELCOME,
    STATE_ALL_ENTRIES,
    STATE_ABOUT,

    STATE_DEBUG,
};

static void state_welcome_start();
static void state_welcome_hnd(enum MENU_INPUT input);

static void state_all_entries_start(uint8_t reset);
static void state_all_entries_hnd(enum MENU_INPUT input);

static void state_debug_start();
static void state_debug_hnd(enum MENU_INPUT input);

static volatile enum MENU_STATE g_state;

void MENU_init()
{
    LCD_init();
    state_welcome_start();
}

void MENU_updatestate(enum MENU_INPUT input)
{
    // propagate input to state handler
    switch (g_state) {
        case STATE_WELCOME:
            state_welcome_hnd(input);
           break; 
        case STATE_ALL_ENTRIES:
            state_all_entries_hnd(input);
            break;
        case STATE_ABOUT:
            state_all_entries_start(0);
            break;
        case STATE_DEBUG:
            state_debug_hnd(input);
            break;
    }
}

static void state_welcome_start()
{
    g_state = STATE_WELCOME;
    LCD_writeInstr(LCD_INSTR_clearDisplay);
    LCD_printAt(2, "> ROCK ON! <");
}

static void state_welcome_hnd(enum MENU_INPUT input)
{
    switch (input) {
        case MENU_INPUT_OK:
            state_all_entries_start(1);
            break;
        case MENU_INPUT_BACK:
            state_debug_start();
            break;
        case MENU_INPUT_UP:
        case MENU_INPUT_DOWN:
            break;
    }
}

#define NUM_ENTRIES 6

static const char *state_all_entries_entries[] = {
    "> About",
    "> Distortion",
    "> Tremolo",
    "> Flanger",
    "> Echo",
    "> Compression"
};

static int8_t state_all_entries_pos;

static void state_all_entries_start(uint8_t reset)
{
    g_state = STATE_ALL_ENTRIES;
    LCD_writeInstr(LCD_INSTR_clearDisplay);
    LCD_printAt(3, "---Menu---");
    if (reset)
        state_all_entries_pos = 0;
    LCD_printAt(LCD_INSTR_nextLine, state_all_entries_entries[state_all_entries_pos]);
}

static void state_all_entries_hnd(enum MENU_INPUT input)
{
    switch (input) {
        case MENU_INPUT_OK:
            // TODO
            if (state_all_entries_pos == 0) {
                g_state = STATE_ABOUT;
                LCD_writeInstr(LCD_INSTR_clearDisplay);
                LCD_printAt(0, "   Constantin  ");
                LCD_printAt(LCD_INSTR_nextLine, "Mihalache 333CB");
            }
            break;
        case MENU_INPUT_BACK:
            state_welcome_start();
            break;
        case MENU_INPUT_UP:
            // TODO
            state_all_entries_pos--;
            if (state_all_entries_pos == -1)
                state_all_entries_pos++;
            LCD_printAt(LCD_INSTR_nextLine, LCD_EMPTY_LINE);
            LCD_printAt(LCD_INSTR_nextLine, state_all_entries_entries[state_all_entries_pos]);
            break;
        case MENU_INPUT_DOWN:
            state_all_entries_pos++;
            if (state_all_entries_pos == NUM_ENTRIES)
                state_all_entries_pos--;
            LCD_printAt(LCD_INSTR_nextLine, LCD_EMPTY_LINE);
            LCD_printAt(LCD_INSTR_nextLine, state_all_entries_entries[state_all_entries_pos]);
            break;
    }
}

static void state_debug_start()
{
    LCD_writeInstr(LCD_INSTR_clearDisplay);
    g_state = STATE_DEBUG;
}

static void state_debug_hnd(enum MENU_INPUT)
{
    state_welcome_start();
}

void MENU_print_value(uint8_t line, const char *text)
{   
    if (g_state != STATE_DEBUG)
        return;
    
    // LCD_printAt(line, LCD_EMPTY_LINE);
    LCD_printAt(line, text);
}